from flair.models import SequenceTagger
from utils.nlu_utils import predict_single, predict_and_annotate
from model.frame import Frame, Slot

"""
ACTS:
    inform/order
    request/menu
    inform/address
    request/price
    request/ingredients
    request/sauce
    inform/phone
    inform/order-complete
    request/time
    request/size
    welcomemsg
    affirm
    inform/delivery
    inform/payment
    request/delivery-price
    bye
    inform/time
    request/drinks
    inform/name
    negate

SLOTS:
    food
    pizza
    size
    address
    quantity
    ingredient
    payment-method
    delivery
    drink
    ingredient/neg
    name
    phone
    sauce
"""

class NaturalLanguageUnderstanding:
    def __init__(self):
        print("\n========================================================")
        print("Models are loading, it may take a moment, please wait...")
        print("========================================================\n")

        self.frame_model = SequenceTagger.load('frame-model-prod/best-model.pt')
        self.slot_model = SequenceTagger.load('slot-model-prod/best-model.pt')

        print("\n========================================================")
        print("Models loaded. NLU system is ready.")
        print("========================================================\n")
    
    def __predict_intention(self, text: str):
        return predict_single(self.frame_model, text.split(), 'frame')

    def __predict_slot(self, text: str):
        anootations = predict_and_annotate(self.slot_model, text.split(), 'slot')
        current_slot = None
        current_slot_value = ""
        slots = []

        for annotation in anootations:
            form = annotation["form"]
            slot = annotation["slot"]

            if slot[0:2] == "B-":
                if current_slot != None:
                    slots.append(Slot(name=current_slot, value=current_slot_value))
                current_slot = slot[2:]
                current_slot_value = form
            elif slot[0:2] == "I-":
                current_slot_value = current_slot_value + " " + form
            elif slot == "O":
                if current_slot != None:
                    slots.append(Slot(name=current_slot, value=current_slot_value))
                    current_slot = None
                    current_slot_value = ""

        if current_slot != None:
            slots.append(Slot(name=current_slot, value=current_slot_value))

        return slots

    def process_input(self, text: str):
        act = self.__predict_intention(text)
        slots = self.__predict_slot(text)
        frame = Frame(source = 'user', act = act, slots = slots)
        return frame